package cs108;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import static java.lang.Math.*;

public final class Main {
    public static void main(String[] args) throws IOException {
        for (int f = 0; f <= 5; f += 1) {
            String fileName = "file" + f + ".bin";
            System.out.println("\n" + fileName + ":");
            int[] freq = byteFrequencies(fileName);
            System.out.println("moyenne: " + average(freq));
            System.out.println("entropie: " + entropy(freq));
            System.out.println("diagramme tige-feuille:");
            List<String> stemPlot = stemPlot(freq);
            for (String s : stemPlot)
                System.out.println(s);
        }
    }

    private static int[] byteFrequencies(String fileName) throws IOException {
        try (InputStream stream = new FileInputStream(fileName)) {
            int[] freq = new int[256];
            int b;
            while ((b = stream.read()) != -1) freq[b] += 1;
            return freq;
        }
    }

    private static double average(int[] freq) {
        int sum = 0, length = 0;
        for (int i = 0; i < freq.length; i += 1) {
            sum += i * freq[i];
            length += freq[i];
        }
        return (double) sum / (double) length;
    }

    private static double entropy(int[] freq) {
        int length = 0;
        for (int f : freq) length += f;

        double hNeg = 0, log2 = log(2);
        for (int f : freq) {
            if (f != 0) {
                double p = (double) f / (double) length;
                hNeg += p * log(p) / log2;
            }
        }
        return -hNeg;
    }

    private static List<String> stemPlot(int[] freq) {
        // calcul du facteur de redimensionnement
        int maxLineLen = 0;
        for (int stem = 0; stem <= freq.length / 10; stem += 1) {
            int lineLen = 3; // 3 = longueur des tiges
            for (int leaf = 0; leaf <= 9; leaf += 1) {
                int i = 10 * stem + leaf;
                if (i >= freq.length) break;
                lineLen += freq[i];
            }
            maxLineLen = max(lineLen, maxLineLen);
        }
        double scale = min(1, 80d / maxLineLen);

        // calcul du diagramme tige et feuilles (redimensionné)
        List<String> lines = new ArrayList<>();
        for (int stem = 0; stem <= freq.length / 10; stem += 1) {
            StringBuilder lineBuilder = new StringBuilder();
            if (stem < 10) lineBuilder.append("0");
            lineBuilder.append(stem).append("|");
            for (int leaf = 0; leaf <= 9; leaf += 1) {
                int i = 10 * stem + leaf;
                if (i >= freq.length) break;
                String leafString = String.valueOf(leaf);
                int leavesCount = (int) (freq[i] * scale);
                lineBuilder.append(leafString.repeat(leavesCount));
            }
            lines.add(lineBuilder.toString());
        }
        return lines;
    }
}
